/*
    Digital circuits emulation library

    Copyright (c) 2004 Alessandro Scotti
    http://www.ascotti.org/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef DCE_9602_H_
#define DCE_9602_H_

/*
    Dual retriggerable resettable one-shot.

    Note: same as 74123, but different pin layout.
*/
class D9602 : public DChip
{
public:
    D9602() : DChip(16) {
        //  1: Cext1 -> external capacitor for one-shot #1
        //  2: Rext1 -> external resistor for one-shot #1
        //  3: CLR1 -> clear one-shot #1, active low
        //  4: B1 -> trigger for one-shot #1, active high
        //  5: A1 -> trigger for one-shot #1, active low
        connectLine(  6, q1_ );
        connectLine(  7, not_q1_ );
        connectLine(  8, DLine::Ground );
        connectLine(  9, not_q2_ );
        connectLine( 10, q2_ );
        // 11: A2 -> trigger for one-shot #2, active low
        // 12: B2 -> trigger for one-shot #2, active high
        // 13: CLR2 -> clear one-shot #1, active low
        // 14: Rext2 -> external resistor for one-shot #2
        // 15: Cext2 -> external capacitor for one-shot #2
        connectLine( 16, DLine::Power );

        q1_ = 0;
        q2_ = 0;
        not_q1_ = 1 ^ q1_;
        not_q2_ = 1 ^ q2_;
        last_input_1_ = 0;
        last_input_2_ = 0;
    }

    void update() {
        DBit input1 = line( 4) | (1 ^ line( 5));
        DBit input2 = line(12) | (1 ^ line(11));

        // TODO!!!

        not_q1_ = 1 ^ q1_;
        not_q2_ = 1 ^ q2_;

        last_input_1_ = input1;
        last_input_2_ = input2;
    }

private:
    DBit    q1_;
    DBit    not_q1_;
    DBit    q2_;
    DBit    not_q2_;
    DBit    last_input_1_;
    DBit    last_input_2_;
};

#endif // DCE_9602_H_
